; SSR.s
; Runs on TM4C1294
; Provide functions that initialize a GPIO pin and turn it on and off.
; Use bit-banded I/O.
; Daniel Valvano
; April 30, 2014

;  This example accompanies the book
;  "Embedded Systems: Introduction to ARM Cortex M Microcontrollers"
;  ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2014
;  Volume 1 Program 4.3, Figure 4.14

;Copyright 2014 by Jonathan W. Valvano, valvano@mail.utexas.edu
;   You may use, edit, run or distribute this file
;   as long as the above copyright notice remains
;THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
;OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
;VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
;OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;For more information about my classes, my research, and my books, see
;http://users.ece.utexas.edu/~valvano/

; solid state relay connected to PN1

GPIO_PORTN1                   EQU 0x40064008
GPIO_PORTN_DIR_R              EQU 0x40064400
GPIO_PORTN_AFSEL_R            EQU 0x40064420
GPIO_PORTN_DEN_R              EQU 0x4006451C
GPIO_PORTN_AMSEL_R            EQU 0x40064528
GPIO_PORTN_PCTL_R             EQU 0x4006452C
SYSCTL_RCGCGPIO_R             EQU 0x400FE608
SYSCTL_RCGCGPIO_R12           EQU 0x00001000  ; GPIO Port N Run Mode Clock Gating Control
SYSCTL_PRGPIO_R               EQU 0x400FEA08
SYSCTL_PRGPIO_R12             EQU 0x00001000  ; GPIO Port N Peripheral Ready

        AREA    |.text|, CODE, READONLY, ALIGN=2
        THUMB
        EXPORT SSR_Init
        EXPORT SSR_On
        EXPORT SSR_Off
        EXPORT SSR_Toggle

;------------SSR_Init------------
; Make PN1 an output, enable digital I/O, ensure alt. functions off.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Init
    ; activate clock for Port N
    LDR R1, =SYSCTL_RCGCGPIO_R      ; R1 = SYSCTL_RCGCGPIO_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #SYSCTL_RCGCGPIO_R12; R0 = R0|SYSCTL_RCGCGPIO_R12
    STR R0, [R1]                    ; [R1] = R0
    ; allow time for clock to stabilize
    LDR R1, =SYSCTL_PRGPIO_R        ; R1 = SYSCTL_PRGPIO_R (pointer)
GPIONinitloop
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ANDS R0, R0, #SYSCTL_PRGPIO_R12 ; R0 = R0&SYSCTL_PRGPIO_R12
    BEQ GPIONinitloop               ; if(R0 == 0), keep polling
    ; set direction register
    LDR R1, =GPIO_PORTN_DIR_R       ; R1 = GPIO_PORTN_DIR_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x02               ; R0 = R0|0x02 (make PN1 out (PN1 built-in LED1))
    STR R0, [R1]                    ; [R1] = R0
    ; set alternate function register
    LDR R1, =GPIO_PORTN_AFSEL_R     ; R1 = GPIO_PORTN_AFSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x02               ; R0 = R0&~0x02 (disable alt funct on PN1)
    STR R0, [R1]                    ; [R1] = R0
    ; set digital enable register
    LDR R1, =GPIO_PORTN_DEN_R       ; R1 = GPIO_PORTN_DEN_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x02               ; R0 = R0|0x02 (enable digital I/O on PN1)
    STR R0, [R1]                    ; [R1] = R0
    ; set port control register
    LDR R1, =GPIO_PORTN_PCTL_R      ; R1 = GPIO_PORTN_PCTL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x000000F0         ; R0 = R0&0xFFFFFF0F (clear bit1 field)
    ADD R0, R0, #0x00000000         ; R0 = R0+0x00000000 (configure PN1 as GPIO)
    STR R0, [R1]                    ; [R1] = R0
    ; set analog mode select register
    LDR R1, =GPIO_PORTN_AMSEL_R     ; R1 = GPIO_PORTN_AMSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x02               ; R0 = R0&~0x02 (disable analog functionality on PN1)
    STR R0, [R1]                    ; [R1] = R0
    BX  LR

;------------SSR_On------------
; Make PN1 high.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_On
    LDR R1, =GPIO_PORTN1            ; R1 = GPIO_PORTN1 (pointer)
    MOV R0, #0x02                   ; R0 = 0x02 (turn on the appliance)
    STR R0, [R1]                    ; [R1] = R0, write to PN1
    BX  LR                          ; return

;------------SSR_Off------------
; Make PN1 low.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Off
    LDR R1, =GPIO_PORTN1            ; R1 = GPIO_PORTN1 (pointer)
    MOV R0, #0x00                   ; R0 = 0x00 (turn off the appliance)
    STR R0, [R1]                    ; [R1] = R0, write to PN1
    BX  LR                          ; return

;------------SSR_Toggle------------
; Toggle PN1.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Toggle
    LDR R1, =GPIO_PORTN1            ; R1 = GPIO_PORTN1 (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (previous value)
    EOR R0, R0, #0x02               ; flip bit 1
    STR R0, [R1]                    ; affect just PN1
    BX  LR

    ALIGN                           ; make sure the end of this section is aligned
    END                             ; end of file
