; SSRTestMain.s
; Runs on TM4C1294
; This is an extremely simple test program to demonstrate that the SSR
; can turn on and off.  Button de-bouncing is not addressed.
; Daniel Valvano
; April 30, 2014

;  This example accompanies the book
;  "Embedded Systems: Introduction to ARM Cortex M Microcontrollers"
;  ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2014
;  Volume 1 Program 4.3, Figure 4.14

;Copyright 2014 by Jonathan W. Valvano, valvano@mail.utexas.edu
;   You may use, edit, run or distribute this file
;   as long as the above copyright notice remains
;THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
;OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
;VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
;OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;For more information about my classes, my research, and my books, see
;http://users.ece.utexas.edu/~valvano/

; built-in negative logic switch connected to PJ0
; solid state relay connected to PN1

GPIO_PORTJ0                   EQU 0x40060004
GPIO_PORTJ_DIR_R              EQU 0x40060400
GPIO_PORTJ_AFSEL_R            EQU 0x40060420
GPIO_PORTJ_PUR_R              EQU 0x40060510
GPIO_PORTJ_DEN_R              EQU 0x4006051C
GPIO_PORTJ_AMSEL_R            EQU 0x40060528
GPIO_PORTJ_PCTL_R             EQU 0x4006052C
SYSCTL_RCGCGPIO_R             EQU 0x400FE608
SYSCTL_RCGCGPIO_R8            EQU 0x00000100  ; GPIO Port J Run Mode Clock Gating Control
SYSCTL_PRGPIO_R               EQU 0x400FEA08
SYSCTL_PRGPIO_R8              EQU 0x00000100  ; GPIO Port J Peripheral Ready

        AREA    |.text|, CODE, READONLY, ALIGN=2
        THUMB
        EXPORT Start
        IMPORT SSR_Init
        IMPORT SSR_On
        IMPORT SSR_Off
        IMPORT SSR_Toggle
Start
    BL  SSR_Init                    ; initialize PN1 and make it output
    ; activate clock for Port J
    LDR R1, =SYSCTL_RCGCGPIO_R      ; R1 = SYSCTL_RCGCGPIO_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #SYSCTL_RCGCGPIO_R8 ; R0 = R0|SYSCTL_RCGCGPIO_R8
    STR R0, [R1]                    ; [R1] = R0
    ; allow time for clock to stabilize
    LDR R1, =SYSCTL_PRGPIO_R        ; R1 = SYSCTL_PRGPIO_R (pointer)
GPIOJinitloop
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ANDS R0, R0, #SYSCTL_PRGPIO_R8  ; R0 = R0&SYSCTL_PRGPIO_R8
    BEQ GPIOJinitloop               ; if(R0 == 0), keep polling
    ; set direction register
    LDR R1, =GPIO_PORTJ_DIR_R       ; R1 = GPIO_PORTJ_DIR_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x01               ; R0 = R0&~0x01 (make PJ0 in (PJ0 built-in SW1))
    STR R0, [R1]                    ; [R1] = R0
    ; set alternate function register
    LDR R1, =GPIO_PORTJ_AFSEL_R     ; R1 = GPIO_PORTJ_AFSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x01               ; R0 = R0&~0x01 (disable alt funct on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; set pull-up register
    LDR R1, =GPIO_PORTJ_PUR_R       ; R1 = GPIO_PORTJ_PUR_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x01               ; R0 = R0|0x01 (enable pull-up on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; set digital enable register
    LDR R1, =GPIO_PORTJ_DEN_R       ; R1 = GPIO_PORTJ_DEN_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    ORR R0, R0, #0x01               ; R0 = R0|0x01 (enable digital I/O on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    ; set port control register
    LDR R1, =GPIO_PORTJ_PCTL_R      ; R1 = GPIO_PORTJ_PCTL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x0000000F         ; R0 = R0&0xFFFFFFF0 (clear bit0 field)
    ADD R0, R0, #0x00000000         ; R0 = R0+0x00000000 (configure PJ0 as GPIO)
    STR R0, [R1]                    ; [R1] = R0
    ; set analog mode select register
    LDR R1, =GPIO_PORTJ_AMSEL_R     ; R1 = GPIO_PORTJ_AMSEL_R (pointer)
    LDR R0, [R1]                    ; R0 = [R1] (value)
    BIC R0, R0, #0x01               ; R0 = R0&~0x01 (disable analog functionality on PJ0)
    STR R0, [R1]                    ; [R1] = R0
    LDR R4, =GPIO_PORTJ0            ; R4 = GPIO_PORTJ0 (pointer)
loop                                ; in this loop, the appliance (PN1) toggles when the switch is released
    BL  SSR_On
waitforpress1                       ; proceed only when the button is pressed
    LDR R0, [R4]                    ; R0 = [R4] (read status of PJ0)
    CMP R0, #0x01                   ; is R0 (PJ0) == 0x01?
    BEQ waitforpress1               ; if so, spin
waitforrelease1                     ; proceed only when the button is released
    LDR R0, [R4]                    ; R0 = [R4] (read status of PJ0)
    CMP R0, #0x01                   ; is R0 (PJ0) != 0x01?
    BNE waitforrelease1             ; if so, spin
    BL  SSR_Off
waitforpress2                       ; proceed only when the button is pressed
    LDR R0, [R4]                    ; R0 = [R4] (read status of PJ0)
    CMP R0, #0x01                   ; is R0 (PJ0) == 0x01?
    BEQ waitforpress2               ; if so, spin
waitforrelease2                     ; proceed only when the button is released
    LDR R0, [R4]                    ; R0 = [R4] (read status of PJ0)
    CMP R0, #0x01                   ; is R0 (PJ0) != 0x01?
    BNE waitforrelease2             ; if so, spin
    B   loop

    ALIGN                           ; make sure the end of this section is aligned
    END                             ; end of file
